from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class CallHistory(Artifact):
    def __post_init__(self) -> None:
        self.name = "Call History"
        self.category = "Call History"
        self.description = "Call History"
        self.web_icon = WebIcon.PHONE_CALL
        self.report_headers = (
            "Timestamp",
            "Phone Number",
            "Name",
            "Answered",
            "Call Type",
            "Call Direction",
            "Call Duration",
            "ISO Country Code",
            "Location",
            "Service Provider",
        )
        self.timeline = True

    @Search("**/CallHistory.storedata*")
    def process(self) -> None:
        for fp in self.found:
            if fp.path.suffix == ".storedata":
                cursor = fp().cursor()
                cursor.execute(
                    """
                    select
                    datetime(ZDATE+978307200,'unixepoch') as TIMESTAMP,
                    ZADDRESS,
                    ZNAME,
                    case ZANSWERED
                        when 0 then 'No'
                        when 1 then 'Yes'
                    end,
                    case ZCALLTYPE
                        when 0 then 'Third-Party App'
                        when 1 then 'Phone'
                        when 8 then 'FaceTime Video'
                        when 16 then 'FaceTime Audio'
                        else ZCALLTYPE
                    end,
                    case ZORIGINATED
                        when 0 then 'Incoming'
                        when 1 then 'Outgoing'
                    end,
                    strftime('%H:%M:%S',ZDURATION, 'unixepoch') as CALLDURATION,
                    upper(ZISO_COUNTRY_CODE),
                    ZLOCATION,
                    ZSERVICE_PROVIDER
                    from ZCALLRECORD
                    """,
                )

                all_rows = cursor.fetchall()
                if all_rows:
                    for row in all_rows:
                        an = str(row["ZADDRESS"])
                        an = an.replace("b'", "")
                        an = an.replace("'", "")
                        row_dict = dict_from_row(row)
                        self.data.append(
                            (row["TIMESTAMP"], an, *tuple(row_dict.values())[2:]),
                        )



import xleapp.helpers.strings as strings

from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class GeodPDPlaceCache(Artifact):
    def __post_init__(self) -> None:
        self.name = "GeoD PD Place Cache"
        self.category = "Locations"
        self.web_icon = WebIcon.MAP_PIN
        self.report_headers = (
            "last access time",
            "requestkey",
            "pdplacehash",
            "expire time",
            "pd place",
        )

    @Search("**/com.apple.geod/PDPlaceCache.db")
    def process(self):
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                SELECT
                datetime('2001-01-01', "lastaccesstime" || ' seconds') as lastaccesstime,
                requestkey,
                pdplacelookup.pdplacehash as pdplacehash,
                datetime('2001-01-01', "expiretime" || ' seconds') as expiretime,
                pdplace
                FROM pdplacelookup
                INNER JOIN pdplaces on pdplacelookup.pdplacehash = pdplaces.pdplacehash
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    pd_place = "".join(
                        f"{pd_row}<br>" for pd_row in set(strings.print(row["pdplace"]))
                    )
                    row_dict = dict_from_row(row)
                    self.data.append((*tuple(row_dict.values())[:4], pd_place))



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class AppleWalletTransactions(Artifact):
    def __post_init__(self) -> None:
        self.name = "Tranactions"
        self.category = "Apple Wallet"
        self.web_icon = WebIcon.DOLLAR_SIGN
        self.report_headers = (
            'Transaction Date',
            'Merchant',
            'Locality',
            'Administrative Area',
            'Currency Amount',
            'Currency Type',
            'Location Date',
            'Latitude',
            'Longitude',
            'Altitude',
            'Peer Payment Handle',
            'Payment Memo',
            'Transaction Status',
            'Transaction Type',
        )
        self.timeline = True

    @Search("**/passes23.sqlite")
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                    SELECT
                    DATETIME(TRANSACTION_DATE + 978307200,'UNIXEPOCH'),
                    MERCHANT_NAME,
                    LOCALITY,
                    ADMINISTRATIVE_AREA,
                    CAST(AMOUNT AS REAL)/100,
                    CURRENCY_CODE,
                    DATETIME(LOCATION_DATE + 978307200,'UNIXEPOCH'),
                    LOCATION_LATITUDE,
                    LOCATION_LONGITUDE,
                    LOCATION_ALTITUDE,
                    PEER_PAYMENT_COUNTERPART_HANDLE,
                    PEER_PAYMENT_MEMO,
                    TRANSACTION_STATUS,
                    TRANSACTION_TYPE
                    FROM PAYMENT_TRANSACTION
                """,
            )

            all_rows = cursor.fetchall()

        if all_rows:
            for row in all_rows:
                row_dict = dict_from_row(row)
                self.data.append(tuple(row_dict.values()))



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class ApplePodcastsEpisodes(Artifact):
    def __post_init__(self) -> None:
        self.name = "Episodes"
        self.category = "Apple Podcasts"
        self.web_icon = WebIcon.PLAY_CIRCLE
        self.report_headers = (
            'Import Date',
            'Metadata Timestamp',
            'Date Last Played',
            'Play State Last Modified',
            'Download Date',
            'Play Count',
            'Author',
            'Title',
            'Subtitle',
            'Asset URL',
            'Web Page URL',
            'Duration',
            'Size',
            'Play State',
        )
        self.timeline = True

    @Search("**/MTLibrary.sqlite")
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                '''
                SELECT
                datetime(ZIMPORTDATE + 978307200, 'unixepoch'),
                CASE ZMETADATATIMESTAMP
                    WHEN 0 THEN ''
                    ELSE datetime(ZMETADATATIMESTAMP + 978307200, 'unixepoch')
                END,
                datetime(ZLASTDATEPLAYED + 978307200, 'unixepoch'),
                datetime(ZPLAYSTATELASTMODIFIEDDATE + 978307200, 'unixepoch'),
                datetime(ZDOWNLOADDATE + 978307200, 'unixepoch'),
                ZPLAYCOUNT,
                ZAUTHOR,
                ZTITLE,
                ZITUNESSUBTITLE,
                ZASSETURL,
                ZWEBPAGEURL,
                ZDURATION,
                ZBYTESIZE,
                ZPLAYSTATE
                FROM ZMTEPISODE
                ORDER by ZMETADATATIMESTAMP
                '''
            )

        all_rows = cursor.fetchall()
        if all_rows:
            for row in all_rows:
                row_dict = dict_from_row(row)
                self.data.append(tuple(row_dict.values()))



import io

import nska_deserialize as nd

from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.utils import deep_get


class CloudKitParticipants(Artifact):
    def __post_init__(self) -> None:
        self.name = "Participants"
        self.category = "CloudKit"
        self.web_icon = WebIcon.USER
        self.description = (
            "CloudKit Participants - CloudKit accounts participating "
            "in CloudKit shares."
        )
        self.report_headers = (
            "Record ID",
            "Email Address",
            "Phone Number",
            "Name Prefix",
            "First Name",
            "Middle Name",
            "Last Name",
            "Name Suffix",
            "Nickname",
        )

    @Search("*NoteStore.sqlite")
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                SELECT Z_PK, ZSERVERSHAREDATA
                FROM
                ZICCLOUDSYNCINGOBJECT
                WHERE
                ZSERVERSHAREDATA NOT NULL
                """,
            )

            all_rows = cursor.fetchall()
            for row in all_rows:
                self.copyfile(
                    row["ZSERVERSHAREDATA"],
                    f"zserversharedata_{str(row['Z_PK'])}.bplist",
                    mode="wb",
                )

                pl = nd.deserialize_plist(io.BytesIO(row["ZSERVERSHAREDATA"]))
                for item in pl:
                    if "Participants" in item:
                        for pt in item.get("Participants"):
                            record_id = deep_get(
                                pt,
                                "UserIdentity",
                                "UserRecordID",
                                "RecordName",
                            )
                            email_address = deep_get(
                                pt,
                                "UserIdentity",
                                "LookupInfo",
                                "EmailAddress",
                            )
                            phone_number = deep_get(
                                pt,
                                "UserIdentity",
                                "LookupInfo",
                                "PhoneNumber",
                            )
                            first_name = deep_get(
                                pt,
                                "UserIdentity",
                                "NameComponents",
                                "NS.nameComponentsPrivate",
                                "NS.givenName",
                            )
                            middle_name = deep_get(
                                pt,
                                "UserIdentity",
                                "NameComponents",
                                "NS.nameComponentsPrivate",
                                "NS.middleName",
                            )
                            last_name = deep_get(
                                pt,
                                "UserIdentity",
                                "NameComponents",
                                "NS.nameComponentsPrivate",
                                "NS.familyName",
                            )
                            name_prefix = deep_get(
                                pt,
                                "UserIdentity",
                                "NameComponents",
                                "NS.nameComponentsPrivate",
                                "NS.namePrefix",
                            )
                            name_suffix = deep_get(
                                pt,
                                "UserIdentity",
                                "NameComponents",
                                "NS.nameComponentsPrivate",
                                "NS.nameSuffix",
                            )
                            nickname = deep_get(
                                pt,
                                "UserIdentity",
                                "NameComponents",
                                "NS.nameComponentsPrivate",
                                "NS.nickname",
                            )

                            self.data.append(
                                (
                                    record_id,
                                    email_address,
                                    phone_number,
                                    name_prefix,
                                    first_name,
                                    middle_name,
                                    last_name,
                                    name_suffix,
                                    nickname,
                                ),
                            )



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class CalendarIdentity(Artifact):
    def __post_init__(self) -> None:
        self.name = "Calendar Identity"
        self.category = "Calendar"
        self.web_icon = WebIcon.CALENDAR
        self.report_headers = ("Display Name", "Address", "First Name", "Last Name")
        self.timeline = True

    @Search("**/Calendar.sqlitedb")
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                    SELECT
                    display_name,
                    address,
                    first_name,
                    last_name
                    from Identity
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    row_dict = dict_from_row(row)
                    self.data.append(tuple(row_dict.values()))



import datetime
import plistlib

from xleapp import Artifact, Search, WebIcon


class BluetoothPaired(Artifact):
    def __post_init__(self) -> None:
        self.name = "Bluetooth Paired"
        self.category = "Bluetooth"
        self.web_icon = WebIcon.BLUETOOTH
        self.report_headers = (
            "Last Seen Time",
            "MAC Address",
            "Name Key",
            "Name",
            "Device Product ID",
            "Default Name",
        )
        self.timeline = True

    @Search("**/com.apple.MobileBluetooth.devices.plist")
    def process(self):
        for fp in self.found:
            pl: dict[str, dict] = plistlib.load(fp())

            for mac_address in pl.keys():
                mac_address_info = pl[mac_address]

                try:
                    lastseen = datetime.datetime.fromtimestamp(
                        int(mac_address_info.get("lastseen")),
                    ).strftime('%Y-%m-%d %H:%M:%S')
                except TypeError:
                    lastseen = None

                userkey = mac_address_info.get("UserNameKey", "")
                nameu = mac_address_info.get("Name", "")
                deviceid = mac_address_info.get("DeviceIDProduct", "")
                defname = mac_address_info.get("DefaultName", "")

                self.data.append(
                    (lastseen, mac_address, userkey, nameu, deviceid, defname),
                )



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.strings import filter_strings_in_file


class DiscordAccounts(Artifact):
    def __post_init__(self):
        self.name = "Discord Accounts"
        self.web_icon = WebIcon.MESSAGE_SQUARE
        self.category = "Discord"

    @Search(
        "*/var/mobile/Containers/Data/Application/*/Documents/mmkv/mmkv.default",
        file_names_only=True,
        return_on_first_hit=False,
    )
    def process(self) -> None:
        for fp in self.found:
            searchlist = [str(string) for string in filter_strings_in_file(fp())]

            cache = ""
            data_list = set()
            for counter, search_string in enumerate(searchlist, start=1):
                if "user_id_cache" in search_string:
                    cache = 'USER_ID_CACHE'
                if "email_cache" in search_string:
                    cache = "EMAIL_CACHE"
                if cache:
                    wf = searchlist[counter].split('"')
                    data_list.add((cache, wf[1]))
                    cache = ""

            self.data.extend(data_list)



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class FacebookMessengerContacts(Artifact):
    def __post_init__(self) -> None:
        self.name = "Contacts"
        self.category = "Facebook Messenger"
        self.report_headers = (
            "User ID",
            "Username",
            "Normalized Username",
            "Profile Pic URL",
            "Is App User",
        )
        self.web_icon = WebIcon.FACEBOOK

    @Search("**/lightspeed-*.db")
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                select
                id,
                name,
                normalized_name_for_search,
                profile_picture_url,
                case is_messenger_user
                    when 0 then ''
                    when 1 then 'Yes'
                end as is_messenger_user
                from contacts
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    row_dict = dict_from_row(row)
                    self.data.append(tuple(row_dict.values()))



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class CalendarList(Artifact):
    def __post_init__(self) -> None:
        self.name = "Calendar List"
        self.category = "Calendar"
        self.web_icon = WebIcon.CALENDAR
        self.report_headers = (
            "Title",
            "Flags",
            "Color",
            "Symbolic Color Name",
            "External ID",
            "Self Identity Email",
        )

    @Search("**/Calendar.sqlitedb")
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                    select
                    title,
                    flags,
                    color,
                    symbolic_color_name,
                    external_id,
                    self_identity_email
                    from Calendar
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    row_dict = dict_from_row(row)
                    self.data.append(tuple(row_dict.values()))



import plistlib

from xleapp import Artifact, Search, WebIcon


class AccountConfiguration(Artifact):
    def __post_init__(self) -> None:
        self.name = "Account Configuration"
        self.category = "Accounts"
        self.web_icon = WebIcon.USER

    @Search("**/com.apple.accounts.exists.plist")
    def process(self):
        for fp in self.found:
            pl = plistlib.load(fp())
            for item in pl.items():
                self.data.append(item)



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class BluetoothPairedLe(Artifact):
    def __post_init__(self) -> None:
        self.name = "Bluetooth Paired LE"
        self.category = "Bluetooth"
        self.web_icon = WebIcon.BLUETOOTH
        self.report_headers = (
            "UUID",
            "Name",
            "Name Origin",
            "Address",
            "Resolved Address",
            "Last Connection Time",
        )

    @Search("**/com.apple.MobileBluetooth.ledevices.paired.db")
    def process(self):
        for fp in self.found:
            cursor = fp().cursor()

            cursor.execute(
                """
                    select
                    Uuid,
                    Name,
                    NameOrigin,
                    Address,
                    ResolvedAddress,
                    LastSeenTime,
                    LastConnectionTime
                    from
                    PairedDevices
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    row_dict = dict_from_row(row)
                    self.data.append(tuple(row_dict.values()))



import json

import magic

from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.utils import deep_get


class DiscordMessages(Artifact):
    def __post_init__(self):
        self.name = "Discord Messages"
        self.category = "Discord"
        self.report_headers = (
            "Timestamp",
            "Edited Timestamp",
            "Username",
            "Bot?",
            "Content",
            "Attachments",
            "User ID",
            "Channel ID",
            "Embedded Author",
            "Author URL",
            "Author Icon URL",
            "Embedded URL",
            "Embedded Script",
            "Footer Text",
            "Footer Icon URL",
            "Source File",
        )
        self.web_icon = WebIcon.MESSAGE_SQUARE

    @Search("*/com.hammerandchisel.discord/fsCachedData/*", return_on_first_hit=False)
    def process(self) -> None:
        for fp in self.found:
            mime = magic.from_file(str(fp()), mime=True)

            if mime != "text/plain":
                continue  # Checks if not text

            with open(fp()) as file_in:
                for json_data in file_in:
                    try:
                        json_parse = json.loads(json_data)

                        if not isinstance(json_parse, list):
                            continue

                        for message in json_parse:
                            username = deep_get(message, "author", "username")
                            userid = deep_get(message, "author", "id")
                            botuser = deep_get(message, "author", "bot")
                            timestamp = deep_get(message, "timestamp")
                            edited_timestamp = deep_get(message, "edited_timestamp")
                            content = deep_get(message, "content")
                            channel_id = deep_get(message, "channel")
                            attachments = {
                                attachment.get("url", "")
                                for attachment in deep_get(message, "attachments")
                            }
                            embeds = deep_get(message, "embeds")

                            embed_metadata = []
                            if embeds:
                                for embed in embeds:
                                    embed_metadata.append(deep_get(embed, "url"))
                                    embed_metadata.append(deep_get(embed, "description"))
                                    embed_metadata.append(
                                        deep_get(embed, "author", "name"),
                                    )
                                    embed_metadata.append(
                                        deep_get(embed, "author", "url"),
                                    )
                                    embed_metadata.append(
                                        deep_get(embed, "author", "icon_url"),
                                    )
                                    embed_metadata.append(
                                        deep_get(embed, "footer", "text"),
                                    )
                                    embed_metadata.append(
                                        deep_get(
                                            embed,
                                            "footer",
                                            "icon_url",
                                        ),
                                    )
                            else:
                                embed_metadata = ['', '', '', '', '', '', '']

                            file_path = fp.path.resolve()

                            self.data.append(
                                (
                                    timestamp,
                                    edited_timestamp,
                                    username,
                                    botuser,
                                    content,
                                    "\n".join(attachments),
                                    userid,
                                    channel_id,
                                    *embed_metadata,
                                    file_path,
                                ),
                            )

                    except TypeError:
                        pass  # File is not JSON



import plistlib

from xleapp import Artifact, Search, WebIcon


class DhcpReceivedList(Artifact):
    def __post_init__(self):
        self.name = "Recevied List"
        self.category = "DHCP"
        self.web_icon = WebIcon.SETTINGS

    @Search("**/private/var/db/dhcpclient/leases/en*")
    def process(self) -> None:
        for fp in self.found:
            pl = plistlib.load(fp())
            for key, value in pl.items():
                if key in [
                    "IPAddress",
                    "LeastLength",
                    "LeastStartDate",
                    "RouterHardwareAddress",
                    "RouterIPAddress",
                    "SSID",
                ]:
                    self.data.append((key, value))



import io

import nska_deserialize as nd

from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.utils import deep_get


class CloudKitSharing(Artifact):
    def __post_init__(self):
        self.name = "CloudKit Note Sharing"
        self.category = "CloudKit"
        self.web_icon = WebIcon.SHARE_2
        self.report_headers = (
            "Record ID",
            "Record Type",
            "Creation Date",
            "Creator ID",
            "Modified Date",
            "Modifier ID",
            "Modifier Device",
        )
        self.description = (
            "CloudKit Note Sharing - Notes information shared via CloudKit."
            "Look up the Record ID in the ZICCLOUDSYYNCINGOBJECT.ZIDENTIFIER column. "
        )

    @Search("*NoteStore.sqlite")
    def process(self) -> None:
        for fp in self.found:

            cursor = fp().cursor()
            cursor.execute(
                """
                    select z_pk, zserverrecorddata
                    from
                    ziccloudsyncingobject
                    where
                    zserverrecorddata not null
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    self.copyfile(
                        row["zserverrecorddata"],
                        f"zserversharedata_{str(row['z_pk'])}.bplist",
                    )

                    pl = nd.deserialize_plist(io.BytesIO(row["zserverrecorddata"]))

                    pl_dict = {}
                    for item in pl:
                        pl_dict.update(item)

                    creation_date = deep_get(pl_dict, "RecordCtime")
                    last_modified_date = deep_get(pl_dict, "RecordMtime")
                    last_modified_id = deep_get(
                        pl_dict,
                        "LastModifiedUserRecordID",
                        "RecordName",
                    )
                    creator_id = deep_get(pl_dict, "CreatorUserRecordID", "RecordName")
                    last_modified_device = deep_get(pl_dict, "ModifiedByDevice")
                    record_type = deep_get(pl_dict, "RecordType")
                    record_id = deep_get(pl_dict, "RecordID", "RecordName")

                    self.data.append(
                        (
                            record_id,
                            record_type,
                            creation_date,
                            creator_id,
                            last_modified_date,
                            last_modified_id,
                            last_modified_device,
                        ),
                    )



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class CalendarItems(Artifact):
    def __post_init__(self) -> None:
        self.name = "Calendar Items"
        self.category = "Calendar"
        self.web_icon = WebIcon.CALENDAR
        self.report_headers = (
            "Start Date",
            "Start Timezone",
            "End Date",
            "End Timezone",
            "All Day?",
            "Summary",
            "Calendar ID",
            "Last Modified",
        )
        self.timeline = True

    @Search("**/Calendar.sqlitedb")
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                    Select
                    DATETIME(start_date + 978307200, 'UNIXEPOCH') as startdate,
                    start_tz,
                    DATETIME(end_date + 978307200, 'UNIXEPOCH') as enddate,
                    end_tz,
                    all_day,
                    summary,
                    calendar_id,
                    DATETIME(last_modified+ 978307200, 'UNIXEPOCH') as lastmod
                    from CalendarItem
                    order by startdate
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    row_dict = dict_from_row(row)
                    self.data.append(tuple(row_dict.values()))



from xleapp import Artifact, Search, WebIcon


class DhcpHotspotClients(Artifact):
    def __post_init__(self) -> None:
        self.name = "Hotspot Clients"
        self.category = "DHCP"
        self.report_headers = ("Hotspot Clients",)
        self.web_icon = WebIcon.SETTINGS

    @Search("**/private/var/db/dhcpd_leases*")
    def process(self) -> None:
        for fp in self.found:
            for line in fp():
                cline = line.strip()
                if cline not in ["{", "}"]:
                    ll = cline.split("=")
                    self.data.append(ll)



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class FacebookMessengerCalls(Artifact):
    def __post_init__(self) -> None:
        self.name = "Calls"
        self.category = "Facebook Messenger"
        self.report_headers = (
            "Timestamp",
            "Sender Name",
            "Sender ID",
            "Call Type",
            "Call Duration/Subtitle",
        )
        self.web_icon = WebIcon.FACEBOOK

    @Search("**/lightspeed-*.db")
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                select
                datetime(thread_messages.timestamp_ms/1000,'unixepoch') as Timestamp,
                contacts.name as 'Sender Name',
                thread_messages.thread_key,
                attachments.title_text as "Call Type",
                attachments.subtitle_text as "Duration/Subtitle"
                from thread_messages
                left join contacts
                    on thread_messages.sender_id = contacts.id
                left join attachments
                    on thread_messages.message_id = attachments.message_id
                where attachments.title_text like 'Audio Call' or attachments.title_text like 'Video Chat'
                order by Timestamp
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    row_dict = dict_from_row(row)
                    self.data.append(tuple(row_dict.values()))



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class CashApp(Artifact):
    def __post_init__(self) -> None:
        self.name = "Transactions"
        self.category = "Cash App"
        self.web_icon = WebIcon.CREDIT_CARD
        self.report_headers = (
            "Transaction Date",
            "Display Name",
            "Cashtag",
            "Account Owner Role",
            "Currency Amount",
            "Transaction State",
            "Transaction State",
        )
        self.timeline = True

    @Search(
        (
            "**private/var/mobile/Containers/Shared/AppGroup/*/"
            "CCEntitySync-api.squareup.com.sqlite"
        ),
    )
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """SELECT
--Unix Epoch timestamp for the transaction display time.
datetime(ZPAYMENT.ZDISPLAYDATE/1000.0,'unixepoch') as "TRANSACTION DISPLAY DATE",

--Full name of the customer as entered into the "First Name" and "Last Name" fields upon application setup.
LTRIM(SUBSTR(CAST(ZCUSTOMER.ZSYNCCUSTOMER AS BLOB), INSTR(CAST(ZCUSTOMER.ZSYNCCUSTOMER AS BLOB), CAST(',"full_name":' AS BLOB)),
instr(CAST(ZCUSTOMER.ZSYNCCUSTOMER AS BLOB), CAST('","is_cash' AS BLOB)) -
instr(CAST(ZCUSTOMER.ZSYNCCUSTOMER AS BLOB), CAST(',"full_name":' AS BLOB))), ',"full_name":"') AS 'CUSTOMER FULL DISPLAY NAME',

--Customer's username created upon application setup.
CASE WHEN INSTR(ZSYNCCUSTOMER, '"cashtag":null') THEN '***NO CASH TAG***' WHEN ZSYNCCUSTOMER LIKE '%C_INCOMING_TRANSFER%' THEN '***NO CASH TAG***' ELSE
LTRIM(SUBSTR(CAST(ZCUSTOMER.ZSYNCCUSTOMER AS BLOB), INSTR(CAST(ZCUSTOMER.ZSYNCCUSTOMER AS BLOB), CAST(',"cashtag":' AS BLOB)),
instr(CAST(ZCUSTOMER.ZSYNCCUSTOMER AS BLOB), CAST('","is_verification' AS BLOB)) -
instr(CAST(ZCUSTOMER.ZSYNCCUSTOMER AS BLOB), CAST(',"cashtag":' AS BLOB))), ',"cashtag":"') END CASHTAG,

-- Description of the role of the user account signed into the CashApp application of the device.
CASE WHEN ZPAYMENT.ZSYNCPAYMENT LIKE '%"RECIPIENT"%' THEN 'RECIPIENT' ELSE 'SENDER' END AS "Account Owner Role",

--Transaction amount sent/received between the account user and customer
printf("$%.2f", LTRIM( SUBSTR(CAST(ZPAYMENT.ZSYNCPAYMENT AS BLOB), INSTR(CAST(ZPAYMENT.ZSYNCPAYMENT AS BLOB), CAST('{"amount":' AS BLOB)),
instr(CAST(ZPAYMENT.ZSYNCPAYMENT AS BLOB), CAST(',"currency' AS BLOB)) -
instr(CAST(ZPAYMENT.ZSYNCPAYMENT AS BLOB), CAST('ount":' AS BLOB)) - 6), '{"amount":') / 100.0) AS 'Transaction Amount',

--State of the transaction. Certain times the user may have to accept or decline a payment or payment request from the sender.
CASE WHEN ZPAYMENT.ZSYNCPAYMENT LIKE '%"COMPLETED"%' THEN 'COMPLETED' WHEN ZPAYMENT.ZSYNCPAYMENT LIKE '%"CANCELED"%' THEN 'CANCELED' ELSE 'WAITING ON RECIPIENT' END AS 'Transaction State',

--Note provided by the sender. Like a memo line on a bank check.
CASE WHEN ZPAYMENT.ZSYNCPAYMENT LIKE '%"note"%' THEN LTRIM( SUBSTR(CAST(ZPAYMENT.ZSYNCPAYMENT AS BLOB), INSTR(CAST(ZPAYMENT.ZSYNCPAYMENT AS BLOB), CAST(',"note":' AS BLOB)),
instr(CAST(ZPAYMENT.ZSYNCPAYMENT AS BLOB), CAST(',"sent' AS BLOB)) -
instr(CAST(ZPAYMENT.ZSYNCPAYMENT AS BLOB), CAST('"note":' AS BLOB))), ',"note":') ELSE '***NO NOTE PRESENT***' END NOTE,

FROM ZPAYMENT
INNER JOIN ZCUSTOMER ON ZCUSTOMER.Z
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    row_dict = dict_from_row(row)
                    self.data.append(tuple(row_dict.values()))



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class FacebookMessengerConversationGroups(Artifact):
    def __post_init__(self) -> None:
        self.name = "Conversation Groups"
        self.category = "Facebook Messenger"
        self.report_headers = (
            "Timestamp (Last Activity)",
            "Thread Key",
            "Thread Participants",
        )
        self.web_icon = WebIcon.FACEBOOK

    @Search("**/lightspeed-*.db")
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                select
                datetime(threads.last_activity_timestamp_ms/1000,'unixepoch'),
                thread_participant_detail.thread_key,
                group_concat(thread_participant_detail.name, ';')
                from thread_participant_detail
                join threads
                    on threads.thread_key = thread_participant_detail.thread_key
                group by thread_participant_detail.thread_key
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    row_dict = dict_from_row(row)
                    self.data.append(tuple(row_dict.values()))



import base64
import re

import defusedxml.ElementTree as ET

from xleapp import Artifact, Search, WebIcon


class DeviceActivator(Artifact):
    def __post_init__(self) -> None:
        self.name = "iOS Device Activator Data"
        self.category = "IOS Build"
        self.web_icon = WebIcon.GIT_COMMIT

    @Search("*private/var/mobile/Library/Logs/mobileactivationd/ucrt_oob_request.txt")
    def process(self) -> None:
        all_lines = ""
        data_list = []

        for fp in self.found:
            for line in fp():
                line = str(line, encoding="utf-8").strip()
                all_lines += line

            found = re.findall(
                (
                    "<key>ActivationInfoXML</key><data>(.*)</data><key>RKCertification"
                    "</key><data>"
                ),
                all_lines,
            )
            base64_message = found[0]

            data = base64.b64decode(base64_message)
            self.copyfile(data, "results.xml")

            xmlfile = self.data_save_folder / "results.xml"
            tree = ET.parse(xmlfile)
            root = tree.getroot()

            for elem in root:
                for elemx in elem:
                    for elemz in elemx:
                        data_list.append(str(elemz.text).strip())

            it = iter(data_list)
            self.data = list(zip(it, it))

            for key, value in self.data:
                if key in (
                    "EthernetMacAddress",
                    "BluetoothAddress",
                    "WifiAddress",
                    "ModelNumber",
                ):
                    self.app.device.update({key: value})



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class ApplePodcastsShows(Artifact):
    def __post_init__(self) -> None:
        self.name = "Shows"
        self.category = "Apple Podcasts"
        self.web_icon = WebIcon.PLAY_CIRCLE
        self.report_headers = (
            'Date Added',
            'Date Last Played',
            'Date Last Updated',
            'Date Downloaded',
            'Author',
            'Title',
            'Feed URL',
            'Description',
            'Web Page URL',
        )
        self.timeline = True

    @Search("**/MTLibrary.sqlite")
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                select
                datetime(ZADDEDDATE + 978307200, 'unixepoch'),
                datetime(ZLASTDATEPLAYED + 978307200, 'unixepoch'),
                datetime(ZUPDATEDDATE + 978307200, 'unixepoch'),
                datetime(ZDOWNLOADEDDATE + 978307200, 'unixepoch'),
                ZAUTHOR,
                ZTITLE,
                ZFEEDURL,
                ZITEMDESCRIPTION,
                ZWEBPAGEURL
                from ZMTPODCAST
            """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    row_dict = dict_from_row(row)
                    self.data.append(tuple(row_dict.values()))



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class GeodApplication(Artifact):
    def __post_init__(self) -> None:
        self.name = "GeoD Application"
        self.category = "Locations"
        self.web_icon = WebIcon.GRID
        self.report_headers = ("Creation Time", "Count ID", "Application")

    @Search("**/com.apple.geod/AP.db")
    def process(self):
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                SELECT count_type, app_id, createtime
                FROM mkcount
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    row_dict = dict_from_row(row)
                    self.data.append(tuple(row_dict.values()))



import plistlib

from xleapp import Artifact, Search, WebIcon
from xleapp.templating.html import NavigationItem


class CellularWirless(Artifact):
    def __post_init__(self) -> NavigationItem:
        self.name = "Celluar Wireless"
        self.category = "Celluar Wireless"
        self.description = "Celluar Wireless"
        self.web_icon = WebIcon.BAR_CHART
        self.report_headers = ("Key", "Value", "Source")

    @Search(
        "**/com.apple.commcenter.plist",
        "**/com.apple.commcenter.device_specific_nobackup.plist",
    )
    def process(self) -> None:
        device_info = self.app.device

        for fp in self.found:
            pl = plistlib.load(fp())
            for key, value in pl.items():
                self.data.append((key, value))
                if key in (
                    "ReportedPhoneNumber",
                    "CDMANetworkPhoneNumberICCID",
                    "imei",
                    "LastKnownICCID",
                    "meid",
                ):
                    if key in ["imei", "meid"]:
                        key = key.upper()
                    device_info.update({key: value})



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class FilesAppICloudSyncNames(Artifact):
    def __post_init__(self) -> None:
        self.name = "iCloud Sync Names"
        self.category = "Files App"
        self.report_headers = ("Name",)
        self.web_icon = WebIcon.FILE_TEXT
        self.description = 'Device names that are able to sync to iCloud Drive.'

    @Search(
        (
            "*private/var/mobile/Library/Application Support/CloudDocs/"
            "session/db/server.db"
        ),
    )
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                SELECT
                name
                FROM
                devices
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    row_dict = dict_from_row(row)
                    self.data.append(tuple(row_dict.values()))



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class FacebookMessengerSecretConversations(Artifact):
    def __post_init__(self) -> None:
        self.name = "Secret Conversations"
        self.category = "Facebook Messenger"
        self.report_headers = (
            "Timestamp",
            "Thread Key",
            "Sender Name",
            "Message (Encrypted)",
            "Attachment (Encrypted)",
        )

        self.web_icon = WebIcon.FACEBOOK

    @Search("**/lightspeed-*.db")
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                 select
                datetime(secure_messages.timestamp_ms/1000,'unixepoch') as 'Timestamp',
                secure_messages.thread_key,
                contacts.name as 'Sender',
                secure_messages.text,
                secure_messages.secure_message_attachments_encrypted
                from secure_messages
                left join contacts
                    on secure_messages.sender_id = contacts.id
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    row_dict = dict_from_row(row)
                    self.data.append(tuple(row_dict.values()))



import io
import plistlib

import nska_deserialize as nd

from xleapp import Artifact, Search, WebIcon


class ApplicationState(Artifact):
    def __post_init__(self) -> None:
        self.name = "Application State"
        self.category = "Installed Apps"
        self.web_icon = WebIcon.PACKAGE
        self.report_headers = ("Bundle ID", "Bundle Path", "Sandbox Path")
        self.description = "Application State DB"

    @Search("**/applicationState.db")
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                select ait.application_identifier as ai, kvs.value as compat_info,
                (SELECT kvs.value from kvs left join application_identifier_tab on application_identifier_tab.id = kvs.application_identifier
                left join key_tab on kvs.key = key_tab.id
                WHERE key_tab.key="XBApplicationSnapshotManifest" and kvs.key = key_tab.id
                and application_identifier_tab.id = ait.id
                ) as snap_info
                from kvs
                left join application_identifier_tab ait on ait.id = kvs.application_identifier
                left join key_tab on kvs.key = key_tab.id
                where key_tab.key="compatibilityInfo"
                order by ait.id
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    plist_file_object = io.BytesIO(row["compat_info"])
                    if row["compat_info"].find(b"NSKeyedArchiver") == -1:
                        plist = plistlib.load(plist_file_object)
                    else:
                        try:
                            plist = nd.deserialize_plist(plist_file_object)
                        except (
                            nd.DeserializeError,
                            nd.biplist.NotBinaryPlistException,
                            nd.biplist.InvalidPlistException,
                            nd.plistlib.InvalidFileException,
                            nd.ccl_bplist.BplistError,
                            ValueError,
                            TypeError,
                            OSError,
                            OverflowError,
                        ) as ex:
                            self.log(
                                message=(
                                    f"Failed to read plist for {row['ai']}, "
                                    f"error was: {str(ex)}"
                                ),
                            )
                    if plist:
                        if isinstance(plist, dict):
                            bundleIdentifier = plist.get("bundleIdentifier", "")
                            bundlePath = plist.get("bundlePath", "")
                            sandboxPath = plist.get("sandboxPath", "")
                            self.data.append((bundleIdentifier, bundlePath, sandboxPath))
                        else:
                            self.log(
                                message=(
                                    f"For {row['ai']} Unexpected type "
                                    f"{str(type(plist))} found as plist root, "
                                    "can't process"
                                ),
                            )
                    else:
                        self.log(
                            message=f"For {str(row['ai'])}, plist could not be read!",
                        )



from xleapp import Artifact, Search, WebIcon
from xleapp.helpers.db import dict_from_row


class FacebookMessengerChats(Artifact):
    def __post_init__(self) -> None:
        self.name = "Chats"
        self.category = "Facebook Messenger"
        self.web_icon = WebIcon.FACEBOOK
        self.report_headers = (
            "Timestamp",
            "Sender Name",
            "Sender ID",
            "Message",
            "Attachment",
            "Attachment Name",
            "Attachment Size",
            "Title Text",
        )

    @Search("**/lightspeed-*.db")
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
                select
                datetime(thread_messages.timestamp_ms/1000,'unixepoch') as Timestamp,
                contacts.name as 'Sender Name',
                thread_messages.thread_key,
                thread_messages.text as 'Message',
                case thread_messages.has_attachment
                    when NULL then ''
                    when 1 then 'Yes'
                end as Attachment,
                attachments.filename as 'Attachment Name',
                attachments.filesize as 'Attachment Size',
                attachment_items.title_text
                from thread_messages
                left join contacts
                    on thread_messages.sender_id = contacts.id
                left join attachments
                    on thread_messages.message_id = attachments.message_id
                left join attachment_items
                    on thread_messages.message_id = attachment_items.message_id
                where attachment_items.title_text IS NULL or attachment_items.title_text like 'Location sharing ended'
                order by Timestamp
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    row_dict = dict_from_row(row)
                    self.data.append(tuple(row_dict.values()))



import datetime

from xleapp import Artifact, Search, WebIcon


class FilesAppIcloudServerItems(Artifact):
    def __post_init__(self) -> None:
        self.name = "iCloud Server Items"
        self.category = "Files App"
        self.report_headers = ("Birthtime", "Filename", "Version Modified Time")
        self.web_icon = WebIcon.FILE_TEXT

    @Search(
        (
            "*private/var/mobile/Library/Application Support/CloudDocs/"
            "session/db/server.db"
        ),
    )
    def process(self) -> None:
        for fp in self.found:
            cursor = fp().cursor()
            cursor.execute(
                """
               SELECT
                item_birthtime,
                item_filename,
                version_mtime
                FROM
                server_items
                """,
            )

            all_rows = cursor.fetchall()
            if all_rows:
                for row in all_rows:
                    birth_time = datetime.datetime.fromtimestamp(row["item_birthtime"])
                    version_m_time = datetime.datetime.fromtimestamp(row["version_mtime"])
                    self.data.append((birth_time, row["item_filename"], version_m_time))



from xleapp import Artifact, Search, WebIcon


class ConnectedDevices(Artifact):
    def __post_init__(self) -> None:
        self.name = "Connected Devices"
        self.category = "Connected to"
        self.web_icon = WebIcon.ZAP
        self.report_headers = ("User & Computer Names",)

    @Search("**/iTunes_Control/iTunes/iTunesPrefs")
    def process(self) -> None:
        for fp in self.found:
            data = fp().read()
            userComps = ""

            self.log(
                message=(
                    "-> Data being interpreted for FRPD is of type: " f"{str(type(data))}"
                ),
            )

            byteArr = bytearray(data)
            userByteArr = bytearray()

            magicOffset = byteArr.find(b"\x01\x01\x80\x00\x00")
            magic = byteArr[magicOffset : magicOffset + 5]

            flag = 0

            if magic == b"\x01\x01\x80\x00\x00":
                self.log(
                    message=(
                        "-> Found magic bytes in iTunes Prefs FRPD... Finding Usernames "
                        "and Desktop names now"
                    ),
                )
                for offset in range(int(magicOffset + 92), len(data)):
                    if (data[offset]) == 0:
                        offset = int(magicOffset) + 157
                        if userByteArr.decode() == "":
                            continue
                        else:
                            if flag == 0:
                                userComps += f"{userByteArr.decode()} - "
                                flag = 1
                            else:
                                userComps += f"{userByteArr.decode()}\n"
                                flag = 0
                            userByteArr = bytearray()
                            continue
                    else:
                        char = data[offset]
                        userByteArr.append(char)
            self.data.append((userComps,))



import json

from xleapp import Artifact, Search, WebIcon


class DiscordManifest(Artifact):
    def __post_init__(self) -> None:
        self.name = "Discord Manifest"
        self.category = "Discord"
        self.web_icon = WebIcon.FILE_TEXT

    @Search(
        (
            "*/private/var/mobile/Containers/Data/Application/*/Documents/"
            "RCTAsyncLocalStorage_V1/manifest.json"
        ),
    )
    def process(self) -> None:
        for fp in self.found:
            for json_data in fp():
                json_parse = json.loads(json_data)

            for key, value in json_parse.items():
                self.data.append((key, value))



import datetime
import plistlib

from xleapp import Artifact, Search, WebIcon


class GoogleMapsCacheRoutes(Artifact):
    def __post_init__(self) -> None:
        self.name = "Google Maps Cache Routes"
        self.category = "Locations"
        self.description = "Google Maps Cache Routes"
        self.web_icon = WebIcon.MAP_PIN
        self.report_headers = ("Timestamp", "Latitude", "Longitude", "Source File")
        self.timeline = True

    @Search(
        "**/Library/Application Support/CachedRoutes/*.plist",
        return_on_first_hit=False,
    )
    def process(self) -> None:
        for fp in self.found:
            timestamp_from_file_name = int(fp.path.stem)
            datatime_time = datetime.datetime.fromtimestamp(
                timestamp_from_file_name / 1000,
            )

            pl = plistlib.load(fp())
            for obj in pl["$objects"]:
                lat = obj["_coordinateLat"]
                lon = obj["_coordinateLong"]
                self.data.append((datatime_time, lat, lon, str(fp.path)))
